﻿#Start-LogShark
#Version V0.1 beta
#Klaus Salger, AXOQUENT GmbH, 06.2010

#SYNOPSIS
#Starts LogShark with a generated configuration file.

#SYNTAX
#Start-LogShark [–ComputerName] <computername> [[-LogSharkConfigTemplate] <LogSharkConfigTemplate>] [[-WrapperConfigFile] <WrapperConfigFile>] [-<CustomVariable> <CustomValue>]*

#DETAILED DESCRIPTION
#Start-LogShark will replace tags in a LogShark configuration file template and run LogShark with the generated configuration file. The computer that LogShark will connect to is given via command line together with optional parameters that allow the setting of config files and custom variables to be replaced in the template.
#The template file is the file to be used for LogShark config file generation. It may contain various tags that will be replaced by variables. Variables may come from the wrapper config file or command line parameters. With the paths to be monitored being the most important variable.
#More than one path per tag may be configured in the WrapperConfigFile. The first accessible path will be used for the replacement. So language or OS version dependent paths may be handled, e.g.  „C:\programme\gemeinsame dateien“ on German and „c:\program files\common files“ on English operating systems.

#PARAMETERS
#-ComputerName <computername>
# The name of the computer that hosts the log directories. May be any name that is accepted in a UNC path.
# Mandatory.
# Named or positional parameter.

#-LogSharkConfigTemplate <LogSharkConfigTemplate>
# The LogShark template file to be used.
# May be a full or relative path or just a filename.
# If this is just a filename it will be loaded from the scripts directory.
# Optional, default=“LogSharkTemplate.lsc” (will look for the file in the same directory as the script).
# Named or positional parameter.

#-WrapperConfigFile <WrapperConfigFile>
# The wrapper configuration file to be used.
# May be a full or relative path or just a filename.
# If this is a filename it will be loaded from the scripts directory.
# Optional, default=“Start-LogShark.xml” (will look for the file in the same directory as the script).
# Named or positional parameter.

#-CustomTag <CustomValue>
# A tag that should be replaced by a the value with the parameter name defining the tag and the actual parameter defining the value.
# It defines a variable like a SimpleVar in the wrapper config xml but lets the user determine the tag-value pairs on the command line.
# Custom tags will be replaced in the template before replacing the tags defined in the wrapper config file.
# So defaults may be defined in the wrapper config file but changed via command line if necessary.
# More than one CustomTag and Customvalue may be used.
# Optional
# Named parameter.


#EXAMPLES
#Example 1
# Powershell –file Start-LogShark –ComputerName Comp0815
# Will start powershell form a command prompt, starting Start-LogShark to monitor directories on the computer “Comp0815”.
# The standard configuration files for the template and the wrapper xml will be used.

#Example 2
#.\Start-LogShark –Computername Comp0816
# – LogSharkConfigTemplate “c:\program files\logshark\enteoClient.lsc”
# -WrapperConfigFile “c:\program files\logshark\enteoWrapper.xml”
# -HoursBack 12 –keepOpen True
# Will start the applet from within powershell, starting to monitor directories on the computer “Comp0816”.
# The configuration files given on the command line will be used. The tag “#HoursBack#” will be replaced by “12”, the tag “KeepOpen” will be replaced by “True”.

# get standard parmeters, eventually setting defaults
param (
	$ComputerName = $(read-host -prompt "Computername"),
	$LogSharkConfigTemplate = "LogSharkTemplate.lsc" ,
	$WrapperConfigFile = "Start-LogShark.xml"
)

# get custom parameters, read into a hash table
# create an empty hash table
$CustomVars = @{}
foreach ($arg in $args) {
	$arg = [string]$arg
	if ($arg[0] -eq "-") {
		$CustomVarsHash = $arg.TrimStart("-")
	}
	else {
		If ($CustomVarsHash -ne $null) {
			$CustomVars[$CustomVarsHash] = $arg
		}
	}
}

# get the current date+time - will serve as part of the generated config filename - format string may be modified but needs to be able to serve as part of a filename
$DateTimeString = Get-Date -Format "yy-MM-dd_hh-mm-ss"

# path of this script
$MyPath = Split-Path -Parent $MyInvocation.MyCommand.Path

# prepend this script's path if we didn't get a path but just a filename
If (!($WrapperConfigFile.Contains("\"))){
	$WrapperConfigFile = $MyPath + "\" + $WrapperConfigFile
}
If (!($LogSharkConfigTemplate.Contains("\"))) {
	$LogSharkConfigTemplate = $MyPath + "\" + $LogSharkConfigTemplate
}

$ComputerUNC = "\\" + $ComputerName

# check if the computer may be reached
$WMIFilter="Address='" + $ComputerName + "'"
$PingStatus = Get-WmiObject -Class "Win32_PingStatus" -Filter $WMIFilter
If ($PingStatus.PrimaryAddressResolutionStatus -eq 0) {
	"Found Computer '" + $ComputerName + "'"
} Else {
	"Computer '" + $ComputerName + "' not found - aborting"
	exit
}

If (Test-Path -Path $LogSharkConfigTemplate) {
	"Using LogShark config template file: '" + $LogSharkConfigTemplate + "'"
	# read in the template file
	$LSConfig = get-content ($LogSharkConfigTemplate)
} Else {
	"LogShark config template file '" + $LogSharkConfigTemplate + "' not found - aborting"
	exit
}

If (Test-Path -Path $WrapperConfigFile) {
	"Using wrapper config file: '" + $WrapperConfigFile + "'"
	# read the whole XML into a variable and make it an XML object
	[XML] $ConfigXML = get-content ($WrapperConfigFile)
} Else {
	"Wrapper config file '" + $WrapperConfigFile + "' not found - aborting"
	exit
}

# write modified config file to the user's temp dir - the current date+time prevents collisions
$TempDir = $Env:TEMP
$TempConfigFile = $TempDir + "\LogSharkConfig." + $DateTimeString + ".lsc"
"Generating LogShark temporary config file: '" + $TempConfigFile + "'"

# loop throuh all tag value pairs and replace the tags by the values in the generated config
# this is the first step so replacing the tags here will prevent them from being replaced later by variables defined in the wrapper config file
foreach ($VarKey in $CustomVars.Keys) {
	$VarTag = "#" + $VarKey + "#"
	$VarTag + " - " + $CustomVars[$VarKey]
	$LSConfig = $LSConfig -replace($VarTag,$CustomVars[$VarKey])
}

# loop through all path groups and all containig paths 
$PathGroupList = $ConfigXML.SelectNodes("WrapperConfig/Paths/Pathgroup")
foreach ($PathGroup in $PathGroupList) {
	# the path group tag in the config file will be replaced by the target path 
	$PathGroupTag = "#" + $PathGroup.Tag + "#"
	"Tag: " + $PathGroupTag
	
	$PathList = $PathGroup.SelectNodes("Path")
	foreach ($DirPath in $PathList) {
		# replace the #computer# tag in the path by the computer name
		$PathGroupPath = $DirPath."#text" -replace("#computer#",$ComputerUNC)
		# check if the path may be found
		if (Test-Path -Path $PathGroupPath) {
			# the path exists, so use it to replace the path group tag in the config file (don't test the rest of the pathes)
			" - Path: using " + $PathGroupPath
			$LSConfig = $LSConfig -replace($PathGroupTag,$PathGroupPath)
			break
		}
		ELSE {
			" - Path: NOT found " + $PathGroupPath
		}
	}
}

# loop through all simple variable definitions 
$VarList = $ConfigXML.SelectNodes("WrapperConfig/SimpleVars/Var")
foreach ($Var in $VarList) {
	# the var tag in the config file will be replaced by the var value 
	$VarTag = "#" + $Var.Tag + "#"
	"Tag: " + $VarTag
	$VarValue = $Var.Value
	"Value: " + $VarValue

	$LSConfig = $LSConfig -replace($VarTag,$VarValue)
}


# write generated config file using ANSI code
Out-File -FilePath $TempConfigFile -encoding "Default" -inputObject $LSConfig

# read command to be executed
$CommandTemplate = $ConfigXML.WrapperConfig.Config.Command
# replace tags in the command template string
$ProgDir = $Env:PROGRAMFILES
$Command = $CommandTemplate -replace ("#PROGRAMFILES#",$ProgDir)
$Command = $Command -replace ("#ResultConfigFile#",$TempConfigFile)

# run command
"Excuting command: " + $Command
$Command="&" + $Command
Invoke-expression $Command
